@@ -2,43 +2,84 @@
 #include "InitData.h"
 #include "dCalc.h"
 #include "error.h"
+#include "consts.h"
 #include <time.h>
 #include <math.h>
+#include <fstream>
+#include <iostream>
 
 
 
-void ViterbiImpl(int tauPara, int JPara,
-				 int MPara, double dPara[], double pPara[], double piPara[], double
-pdfPara[],
-				 int hiddenStatesPara[])
+void ViterbiImpl(int tauPara, int JPara, int MPara, 
+				 double dPara[], double pPara[], double piPara[], double pdfPara[], int hiddenStatesPara[])
 {
 	int i, j, k = 0, k_alt, t, u, dummyInt = 0;
 	double Observ, x = 0;
 	bool first, first_i, first_alpha;
 
+	// output all function parameters to file
+	if (run_mode == STORE_VIT) {
+		ofstream ofs(PARA_FNAME.c_str());
+		if (!ofs) {
+			cerr << "unable to open file: " << PARA_FNAME << endl;
+			exit(0);
+		}
+		
+		// output tau, J, and M
+		ofs << 0 << endl;   // dummy output for CensoringPara
+		ofs << tauPara << endl; 
+		ofs << JPara << endl; 
+		ofs << MPara << endl << endl; 
+		
+		// output d
+		for (int j = 0; j < JPara; ++j) {
+			for (int m = 0; m < MPara; ++m) {
+				ofs << j << " " << m << " " << dPara[j * MPara + m] << endl;
+			}
+		}
+		ofs << endl;
+		 
+		// output p		
+		for (int j = 0; j < JPara; ++j) {
+			for (int k = 0; k < JPara; ++k) {
+				ofs << j << " " << k << " " << pPara[j * JPara + k] << endl;
+			}
+		}
+		ofs << endl;
+
+		// output pi
+		for (int j = 0; j < JPara; ++j) {
+			ofs << j << " " << piPara[j] << endl;
+		}
+		ofs << endl;
+		
+		// output pdf
+		for (int j = 0; j < JPara; ++j) {
+			for (int t = 0; t < tauPara; ++t) {
+				ofs << j << " " << t << " " << pdfPara[j * tauPara + t] << endl;
+			}
+		}
+		ofs << endl;
+	}
+
 	InitParaAndVar(dummyInt, tauPara, JPara, MPara, dPara, pPara, piPara, pdfPara);
 
 	CalcStoreD();
 
-	for (t = 0; t <= tau - 1; t++)
-	{
-		for (j = 0; j <= J - 1; j++)
-		{
+	for (t = 0; t <= tau - 1; t++) {
+		for (j = 0; j <= J - 1; j++) {
 			Observ = 0;
 			first_alpha = true;
-			for (u = 1; u <= min(t, M); u++)
-			{
+			for (u = 1; u <= min(t, M); u++) {
 				first_i = true;
 				for (i = 0; i <= J - 1; i++)
 					if (i != j)
-						if ((log(p[i][j]) + alpha[i][t - u] > x) || first_i)
-						{
+						if ((log(p[i][j]) + alpha[i][t - u] > x) || first_i) {
 							x = log(p[i][j]) + alpha[i][t - u];
 							k = i;
 							first_i = false;
 						}
-				if ((Observ + log(d[j][u]) + x > alpha[j][t]) || first_alpha)
-				{
+				if (first_alpha || (Observ + log(d[j][u]) + x > alpha[j][t])) {
 					alpha[j][t] = Observ + log(d[j][u]) + x;
 					maxU[j][t] = u;
 					maxI[j][t] = k;
@@ -46,13 +87,13 @@ pdfPara[],
 				}
 				Observ += log(pdf[j][t - u]);
 			}
-			if (t + 1 <= M)
-				if ((Observ + log(d[j][t + 1] * pi[j]) > alpha[j][t]) || first_alpha)
-				{
+			if (t + 1 <= M) {
+				if (first_alpha || (Observ + log(d[j][t + 1] * pi[j]) > alpha[j][t])) {
 					alpha[j][t] = Observ + log(d[j][t + 1] * pi[j]);
 					maxU[j][t] = -1;
 					maxI[j][t] = -1;
 				}
+			}
 			alpha[j][t] += log(pdf[j][t]);
 		}
 	}
@@ -118,7 +159,7 @@ pdfPara[],
 		hiddenStates[i] = k;
 
 	// Save parameters
-	for (t = 0; t <= tau; t++)
+	for (t = 0; t < tau; t++)
 	{
 		hiddenStatesPara[t] = hiddenStates[t];		
 	}
