@@ -5,7 +5,7 @@
 	#include <assert.h>
 	#include "matrix.h"
 
-	template<class T>inline T*** cube(int Depth, int Rows, int Columns)
+	template<class T>inline T*** cube(long Depth, long Rows, long Columns)
 	{
 		T ***m = matrix<T*>(Depth, Rows);
 		if (m == NULL)
@@ -33,7 +33,7 @@
 	template<class T>inline void free_cube(T*** Cube)
 	{
 		delete[](Cube[0][0]);
-		delete[](Cube);
+		free_matrix(Cube);
 	}
 
 #endif
